@@ -1093,7 +1093,7 @@ void FcitxInstanceSwitchIMByIndex(FcitxInstance* instance, int index)
     else {
         if (ic)
             FcitxInstanceSetLocalIMName(instance, ic, NULL);
-        FcitxInstanceSwitchIMInternal(instance, index, true, true);
+        FcitxInstanceSwitchIMInternal(instance, index, true, true, true);
         FcitxInstanceShowInputSpeed(instance);
 
         if (FcitxInstanceGetCurrentState(instance) != IS_ACTIVE) {
@@ -1115,20 +1115,32 @@ void FcitxInstanceUnregisterIM(FcitxInstance* instance, const char* name)
 
 void
 FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index,
-                              boolean skipZero, boolean updateGlobal)
+                              boolean skipZero, boolean updateGlobal, boolean userSwitchIM)
 {
     UT_array* imes = &instance->imes;
     const int iIMCount = utarray_len(imes);
 
-    FcitxInstanceCleanInputWindow(instance);
-    FcitxInstanceResetInput(instance);
-    FcitxUIUpdateInputWindow(instance);
-
     FcitxIM* lastIM, *newIM;
 
     /* set lastIM */
     lastIM = fcitx_array_eltptr(imes, instance->iIMIndex);
 
+    if (lastIM) {
+
+        if (userSwitchIM) {
+            if (lastIM->OnClose) {
+                lastIM->OnClose(lastIM->klass, CET_SwitchIM);
+            }
+        }
+        if (lastIM->Save) {
+            lastIM->Save(lastIM->klass);
+        }
+    }
+
+    FcitxInstanceCleanInputWindow(instance);
+    FcitxInstanceResetInput(instance);
+    FcitxUIUpdateInputWindow(instance);
+
     /* update instance->iIMIndex start */
     if (index >= iIMCount)
         instance->iIMIndex = iIMCount - 1;
@@ -1148,9 +1160,6 @@ FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index,
     /* set newIM */
     newIM = fcitx_array_eltptr(imes, instance->iIMIndex);
 
-    if (lastIM && lastIM->Save)
-        lastIM->Save(lastIM->klass);
-
     /* lazy load */
     if (newIM && !newIM->initialized) {
         char* name = strdup(newIM->uniqueName);
@@ -1213,6 +1222,15 @@ void FcitxInstanceResetInput(FcitxInstance* instance)
     FcitxInstanceProcessResetInputHook(instance);
 }
 
+FCITX_EXPORT_API
+void FcitxInstanceSendCloseEvent(struct _FcitxInstance* instance, FcitxIMCloseEventType closeEvent)
+{
+    FcitxIM* currentIM = FcitxInstanceGetCurrentIM(instance);
+    if (currentIM && currentIM->OnClose) {
+        currentIM->OnClose(currentIM->klass, closeEvent);
+    }
+}
+
 void FcitxInstanceDoPhraseTips(FcitxInstance* instance)
 {
     UT_array *ims = &instance->imes;
@@ -1424,12 +1442,13 @@ void FcitxInstanceSetLocalIMName(FcitxInstance* instance, FcitxInputContext* ic,
     if (imname)
         ic2->imname = strdup(imname);
 
-    if (ic == FcitxInstanceGetCurrentIC(instance))
-        FcitxInstanceUpdateCurrentIM(instance, false);
+    if (ic == FcitxInstanceGetCurrentIC(instance)) {
+        FcitxInstanceUpdateCurrentIM(instance, false, true);
+    }
 }
 
 /* the "force" to this function is used when the list changed and index is not valid */
-boolean FcitxInstanceUpdateCurrentIM(FcitxInstance* instance, boolean force) {
+boolean FcitxInstanceUpdateCurrentIM(FcitxInstance* instance, boolean force, boolean userSwitchIM) {
     FcitxInputContext* ic = FcitxInstanceGetCurrentIC(instance);
     if (!ic && !force)
         return false;
@@ -1471,7 +1490,7 @@ boolean FcitxInstanceUpdateCurrentIM(FcitxInstance* instance, boolean force) {
     }
 
     if (forceSwtich || targetIMIndex != instance->iIMIndex) {
-        FcitxInstanceSwitchIMInternal(instance, targetIMIndex, skipZero, updateGlobal);
+        FcitxInstanceSwitchIMInternal(instance, targetIMIndex, skipZero, updateGlobal, userSwitchIM);
         return true;
     }
     else
@@ -1507,7 +1526,7 @@ void FcitxInstanceEnableIM(FcitxInstance* instance, FcitxInputContext* ic, boole
         break;
     }
 
-    FcitxInstanceUpdateCurrentIM(instance, false);
+    FcitxInstanceUpdateCurrentIM(instance, false, false);
     instance->input->keyReleased = KR_OTHER;
 }
 
@@ -1622,7 +1641,7 @@ void FcitxInstanceChangeIMStateWithKey(FcitxInstance* instance, FcitxInputContex
         break;
     }
 
-    FcitxInstanceUpdateCurrentIM(instance, false);
+    FcitxInstanceUpdateCurrentIM(instance, false, !withSwitchKey);
 }
 
 /**
@@ -1646,10 +1665,7 @@ void FcitxInstanceChangeIMStateInternal(FcitxInstance* instance, FcitxInputConte
     FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
     ic2->switchBySwitchKey = withSwitchKey;
     if (ic == instance->CurrentIC) {
-        if (objectState == IS_ACTIVE) {
-            FcitxInstanceResetInput(instance);
-        } else {
-            FcitxInstanceResetInput(instance);
+        if (objectState != IS_ACTIVE) {
             FcitxUICloseInputWindow(instance);
         }
     }
@@ -1979,7 +1995,7 @@ void FcitxInstanceUpdateIMList(FcitxInstance* instance)
 
     utarray_free(imList);
 
-    FcitxInstanceUpdateCurrentIM(instance, true);
+    FcitxInstanceUpdateCurrentIM(instance, true, false);
     FcitxInstanceProcessUpdateIMListHook(instance);
 
     if (instance->globalIMName)
